==> sli-plots-0.0.20.tgz: package/addon/components/plot-axes-and-ticks.js <==
import Component from '@glimmer/component';
import { scaleTime, scaleLinear } from "d3-scale";

export default class PlotAxesAndTicksComponent extends Component {
  get xScaleRange() {
    return this.args.dimensions
    ? scaleTime().range([0, this.args.dimensions.width])
    : {}
  }

  get yScaleRange() {
    return this.args.dimensions
    ? scaleLinear().range([this.args.dimensions.height, 0])
      : {}
  }

  makeYTicks(min, max) {
    const { isMultiDataset, data, hasData, dimensions, onlyMinMaxTicks } = this.args;

    if((isMultiDataset && data[0].length === 0) || !hasData) {
      return { yScale: this.yScaleRange.domain ? this.yScaleRange.domain([min, max]) : () => {} }
    }
    const tickCount = dimensions.height < 500 ? 7 : 10;
    const stepSize = this.tickStepSize(max - min, tickCount);
    const maxTick = Math.ceil(max / stepSize) * stepSize;
    const yScale = this.yScaleRange.domain([min, maxTick]);
    return {
      max: maxTick,
      min,
      yScale,
      ticks: Array.from({length: onlyMinMaxTicks ? 2 : (maxTick/stepSize + 1)}, (_, i) => {
        const value = onlyMinMaxTicks
                        ? i === 0 ? min : maxTick
                        : i * stepSize + min
        const fixedValue = value < 1 && value !== 0 ? value.toFixed(2) : value;
        const y = yScale(fixedValue);
        return {
          label: `${fixedValue}`,
          fixedValue,
          y,
          position: `top: ${y}px;`
        }
      })
    }
  }

  get yScaling() {
    if(!this.args.data) return [0,0]
    const dataMax = this.getYMax();
    const dataMin = this.getYMin();
    const yTicksInfo = this.makeYTicks(dataMin, dataMax);
    const yScale = yTicksInfo.yScale;
    return {
      max: yTicksInfo.max ? yTicksInfo.max : dataMax,
      min: dataMin,
      ticks: yTicksInfo.ticks,
      yScale
    }
  }

  get xScaling() {
    const { data, dimensions, sortedTimeRange, isMultiRange, timeInterval } = this.args;
    if(!data || !dimensions) return [0,0]
    const xScale = isMultiRange
      ? sortedTimeRange.map(timeRange => this.makeXScale(timeRange))
      : this.xScaleRange.domain(sortedTimeRange);
    
    const range = isMultiRange ? sortedTimeRange[0] : sortedTimeRange;
    const startTime = range[0].getTime();
    const fixedInterval = timeInterval < 900000 ? 900000 : timeInterval; // cap at 15 min
    const idealTickCount = ((range[1].getTime() - range[0].getTime()) / fixedInterval) + 1; // tick for every interval + origin
    const labelCount = this.xLabelCount(); // number of labels based on dimensions of chart
    const tickDensity = idealTickCount/dimensions.width * 100; // the lower the value, the more spread apart they are
    const tooDense = tickDensity < 6;
    const tickCount = tooDense ? idealTickCount : idealTickCount / 2;
    const labelStep = Math.ceil(tickCount / labelCount);
    
    return {
      xScale,
      ticks: Array.from({length: tickCount}, (_, i) => {
        const value = new Date(i * (tooDense ? fixedInterval : fixedInterval * 2) + startTime)
        const x = isMultiRange ? xScale[0](value) : xScale(value)
        return {
          label: i % labelStep || i > (7/8) * tickCount ? "" : this.formatDate(value, fixedInterval >= 1000*60*60*24),
          x,
          position: `left: ${x}px;`
        }
      })
    }
  }

  makeXScale(range) {
    return this.xScaleRange.domain(range);
  }

  xLabelCount() {
    const width = this.args.dimensions.width;
    if(width < 300) {
      return 3
    } else if (width < 600) {
      return 4
    } else {
      return 6
    }
  }

  getYMax() {
    const { yMax, data, scaleYBasedOnTotalDataset, filteredData, isMultiDataset, isMultiRange, yKey } = this.args;
    if(yMax) { // override with yMax argument for fixed max
      return yMax
    } else if(data) {
      const maxData = scaleYBasedOnTotalDataset ? data : filteredData;
      const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? maxData.flat(1) : maxData;
      return dataset.length > 0 ? Math.max(...dataset.map(d => d[yKey])) : 0;
    } else {
      return 0;
    }
  }

  getYMin() {
    const { y0, yKey, data, filteredData, isMultiDataset, isMultiRange, scaleYBasedOnTotalDataset, autoY0 } = this.args;
    if(y0) return y0;
    const minData = scaleYBasedOnTotalDataset ? data : filteredData;
    const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? minData.flat(1) : minData
    return autoY0 && dataset.length > 0 ? Math.min(...dataset.map(d => d[yKey])) : y0;
  }

  pad(number, padding = -2) {
    return ('0' + number).slice(padding)
  }
  
  formattedTime(date) {
    const hours = date.getHours()
    const nonMilitary = hours > 12 ? hours - 12 : hours;
    const amPm = hours >= 12 ? 'pm' : 'am';
    return `${nonMilitary === 0 ? 12 : nonMilitary}:${this.pad(date.getMinutes())} ${amPm}`;
  }
  
  formatDate(date, hideTime) {
    if(!date || typeof date !== 'object' || !date.getMonth) return ""
    const time = hideTime ? '' : this.formattedTime(date);
    return `${date.getMonth() + 1}-${date.getDate()}-${date.getFullYear()% 100} ${time}`;
  }
  

  tickStepSize(range, targetSteps) {
    const startingStep = range / targetSteps;
     // magnitude
    const magnitude = Math.floor(Math.log(startingStep) / Math.LN10);
    const MagnitudePower = Math.pow(10, magnitude);
     // signif
    let residual = Math.round(startingStep/MagnitudePower);
    if( residual >= 5.0 ) {
      residual = 10.0;
    } else if ( residual >= 2.0 ) {
      residual = 5.0;
    } else if ( residual >= 1.0 ) {
      residual = 2.0;
    }

    return residual * MagnitudePower;
  }
}

==> sli-plots-0.0.23.tgz: package/addon/components/plot-axes-and-ticks.js <==
import Component from '@glimmer/component';
import { scaleTime, scaleLinear } from "d3-scale";

export default class PlotAxesAndTicksComponent extends Component {
  get xScaleRange() {
    return this.args.dimensions
    ? scaleTime().range([0, this.args.dimensions.width])
    : {}
  }

  get yScaleRange() {
    return this.args.dimensions
    ? scaleLinear().range([this.args.dimensions.height, 0])
      : {}
  }

  makeYTicks(originalMin, max) {
    const { isMultiDataset, data, hasData, dimensions, onlyMinMaxTicks } = this.args;
    const min = this.roundMinimum(originalMin, max);
    if((isMultiDataset && data[0].length === 0) || !hasData) {
      return { yScale: this.yScaleRange.domain ? this.yScaleRange.domain([min, max]) : () => {} }
    }

    const tickCount = dimensions.height < 500 ? 7 : 10;
    const stepSize = this.tickStepSize(max - min, tickCount);
    const maxTick = Math.ceil(max / stepSize) * stepSize;
    const yScale = this.yScaleRange.domain([min, maxTick]);
    return {
      max: maxTick,
      min,
      yScale,
      ticks: Array.from({length: onlyMinMaxTicks ? 2 : Math.ceil((maxTick - min)/(stepSize + 1))}, (_, i) => {
        const value = onlyMinMaxTicks
                        ? i === 0 ? min : maxTick
                        : i * stepSize + min
        const fixedValue = value < 1 && value !== 0 ? value.toFixed(2) : value;
        const y = yScale(fixedValue);
        return {
          label: `${fixedValue}`,
          fixedValue,
          y,
          position: `top: ${y}px;`
        }
      })
    }
  }

  // This method returns a new minimum value that is rounded down based on
  // the magnatude of of min max for a nice rounded value that isn't too far off
  // from the lowest value
  roundMinimum(min, max) {
    if(min === 0) return min;

    const diff = max - min;

    if(diff < 20) return Math.floor(min);
    if(diff < 100) return Math.floor(min / 10) * 10;
    if(diff < 2000) return Math.floor(min / 100) * 100;

    return Math.floor(min / 1000) * 1000;
  }

  get yScaling() {
    if(!this.args.data) return [0,0]
    const dataMax = this.getYMax();
    const dataMin = this.getYMin();
    const yTicksInfo = this.makeYTicks(dataMin, dataMax);
    const yScale = yTicksInfo.yScale;
    return {
      max: yTicksInfo.max ? yTicksInfo.max : dataMax,
      min: dataMin,
      ticks: yTicksInfo.ticks,
      yScale
    }
  }

  get xScaling() {
    const { data, dimensions, sortedTimeRange, isMultiRange, timeInterval } = this.args;
    if(!data || !dimensions) return [0,0]
    const xScale = isMultiRange
      ? sortedTimeRange.map(timeRange => this.makeXScale(timeRange))
      : this.xScaleRange.domain(sortedTimeRange);
    
    const range = isMultiRange ? sortedTimeRange[0] : sortedTimeRange;
    const startTime = range[0].getTime();
    const fixedInterval = timeInterval < 900000 ? 900000 : timeInterval; // cap at 15 min
    const idealTickCount = ((range[1].getTime() - range[0].getTime()) / fixedInterval) + 1; // tick for every interval + origin
    const labelCount = this.xLabelCount(); // number of labels based on dimensions of chart
    const tickDensity = idealTickCount/dimensions.width * 100; // the lower the value, the more spread apart they are
    const tooDense = tickDensity < 6;
    const tickCount = tooDense ? idealTickCount : idealTickCount / 2;
    const labelStep = Math.ceil(tickCount / labelCount);
    
    return {
      xScale,
      ticks: Array.from({length: tickCount}, (_, i) => {
        const value = new Date(i * (tooDense ? fixedInterval : fixedInterval * 2) + startTime)
        const x = isMultiRange ? xScale[0](value) : xScale(value)
        return {
          label: i % labelStep || i > (7/8) * tickCount ? "" : this.formatDate(value, fixedInterval >= 1000*60*60*24),
          x,
          position: `left: ${x}px;`
        }
      })
    }
  }

  makeXScale(range) {
    return this.xScaleRange.domain(range);
  }

  xLabelCount() {
    const width = this.args.dimensions.width;
    if(width < 300) {
      return 3
    } else if (width < 600) {
      return 4
    } else {
      return 6
    }
  }

  getYMax() {
    const { yMax, data, scaleYBasedOnTotalDataset, filteredData, isMultiDataset, isMultiRange, yKey } = this.args;
    if(yMax) { // override with yMax argument for fixed max
      return yMax
    } else if(data) {
      const maxData = scaleYBasedOnTotalDataset ? data : filteredData;
      const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? maxData.flat(1) : maxData;
      return dataset.length > 0 ? Math.max(...dataset.map(d => d[yKey])) : 0;
    } else {
      return 0;
    }
  }

  getYMin() {
    const { y0, yKey, data, filteredData, isMultiDataset, isMultiRange, scaleYBasedOnTotalDataset, autoY0 } = this.args;
    if(y0) return y0;
    const minData = scaleYBasedOnTotalDataset ? data : filteredData;
    const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? minData.flat(1) : minData
    return autoY0 && dataset.length > 0 ? Math.min(...dataset.map(d => d[yKey])) : y0;
  }

  pad(number, padding = -2) {
    return ('0' + number).slice(padding)
  }
  
  formattedTime(date) {
    const hours = date.getHours()
    const nonMilitary = hours > 12 ? hours - 12 : hours;
    const amPm = hours >= 12 ? 'pm' : 'am';
    return `${nonMilitary === 0 ? 12 : nonMilitary}:${this.pad(date.getMinutes())} ${amPm}`;
  }
  
  formatDate(date, hideTime) {
    if(!date || typeof date !== 'object' || !date.getMonth) return ""
    const time = hideTime ? '' : this.formattedTime(date);
    return `${date.getMonth() + 1}-${date.getDate()}-${date.getFullYear()% 100} ${time}`;
  }
  

  tickStepSize(range, targetSteps) {
    const startingStep = range / targetSteps;
     // magnitude
    const magnitude = Math.floor(Math.log(startingStep) / Math.LN10);
    const MagnitudePower = Math.pow(10, magnitude);
     // signif
    let residual = Math.round(startingStep/MagnitudePower);
    if( residual >= 5.0 ) {
      residual = 10.0;
    } else if ( residual >= 2.0 ) {
      residual = 5.0;
    } else if ( residual >= 1.0 ) {
      residual = 2.0;
    }

    return residual * MagnitudePower;
  }
}
